Option Explicit

'*
'* Global variables...
'*

Global Answer As Integer
Global NumberOfPlayers As Integer
Global Movement As Integer

Global Delay As Single

Global Colour() As Integer
Global Direction() As Integer

'*
'* Movement constants...
'*

Global Const GORIGHT = 1
Global Const GOLEFT = 2
Global Const GOUP = 3
Global Const GODOWN = 4

'*
'* General constants lifted from the standard VB file CONSTANTS.TXT...
'*

Global Const MODAL = 1                 '* Make a form modal

Global Const MB_OK = 0                 '* OK button only
Global Const MB_YESNO = 4              '* Yes and No buttons

Global Const MB_ICONSTOP = 16          '* Critical message
Global Const MB_ICONQUESTION = 32      '* Warning query
Global Const MB_ICONEXCLAMATION = 48   '* Warning message
Global Const MB_ICONINFORMATION = 64   '* Information message
                                        
Global Const IDOK = 1                  '* OK button pressed
Global Const IDYES = 6                 '* Yes button pressed
Global Const IDNO = 7                  '* No button pressed

'*************************************************************************
'*                                                                       *
'*                Name: CentreForm                                       *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 10 November 1993                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To centre a form on-screen and make it the top-  *
'*                      most form.                                       *
'*                                                                       *
'*    Passed variables: xForm - The form to operate on                   *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes: None                                             *
'*                                                                       *
'*************************************************************************
'*
Sub CentreForm (xForm As Form)

  xForm.Left = Int((Screen.Width - xForm.Width) / 2)
  xForm.Top = Int((Screen.Height - xForm.Height) / 2)
  xForm.ZOrder

End Sub

'*************************************************************************
'*                                                                       *
'*                Name: DetectCollision                                  *
'*                Type: Function                                         *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 7 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To detect an illegal posion. This could be one   *
'*                      of the Spacoids going out of the playing area or *
'*                      trying to cross a trail.                         *
'*                                                                       *
'*    Passed variables: Xp - The X or horizontal position to check       *
'*                      Yp - The Y or vertical position to check         *
'*                                                                       *
'*  Returned variables: DetectCollision - True if no collision detected  *
'*                                        False otherwise                *
'*                                                                       *
'*               Notes:                                                  *
'*                                                                       *
'*************************************************************************
'*
Function DetectCollision (Xp As Single, Yp As Single) As Integer

  Dim Count As Integer

  Dim Collision As Long
  
  DetectCollision = True
  
  Collision = FrmPlay.BackDrop.Point(Xp, Yp)

  If Collision = -1 Then

'*
'* This Spaceoid is going out of bounds...
'*

    DetectCollision = False
    Exit Function

  End If

  For Count = 1 To NumberOfPlayers

    If Collision = QBColor(Colour(Count)) Then

'*
'* and this one has hit a trail...
'*

      DetectCollision = False
      Exit For

    End If

  Next

End Function

'*************************************************************************
'*                                                                       *
'*                Name: EndPlay                                          *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To show who won - or to be more precise - who    *
'*                      lost!                                            *
'*                                                                       *
'*    Passed variables: Wh - The loser                                   *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes:                                                  *
'*                                                                       *
'*************************************************************************
'*
Sub EndPlay (Wh As Integer)

  CentreForm FrmEnd
  Load FrmEnd

  FrmEnd.Label = "Player " & CStr(Wh) & " - you lose!"
  FrmEnd.Label.ForeColor = QBColor(Colour(Wh) - 8)

  FrmEnd.Show MODAL

  FrmPlay.BackDrop.Refresh

End Sub

'*************************************************************************
'*                                                                       *
'*                Name: Initialise                                       *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To set up all the system variables, put the      *
'*                      Spacoids in their starting positions and start   *
'*                      the game.                                        *
'*                                                                       *
'*    Passed variables: None                                             *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes:                                                  *
'*                                                                       *
'*************************************************************************
'*
Sub Initialise ()

  Dim Position() As Single

  ReDim Position(1 To NumberOfPlayers, 0 To 1)

'*
'* Start all the Spacoids from their own positions around the centre.
'* For a two person game this just to the left and under and just to the
'* right and above the centre point...
'*

  Position(1, 0) = (FrmPlay.BackDrop.ScaleWidth \ 2) + 100
  Position(1, 1) = (FrmPlay.BackDrop.ScaleHeight \ 2) + 100

  Position(2, 0) = (FrmPlay.BackDrop.ScaleWidth \ 2) - 100
  Position(2, 1) = (FrmPlay.BackDrop.ScaleHeight \ 2) - 100

  FrmPlay.BackDrop.DrawWidth = 4

  FrmPlay.BackDrop.PSet (Position(1, 0), Position(1, 1)), QBColor(Colour(1))
  FrmPlay.BackDrop.PSet (Position(2, 0), Position(2, 1)), QBColor(Colour(2))

  DoEvents

'*
'* and set each to move away from its starting position in a logical
'* direction at a set speed...
'*

  Direction(1) = GORIGHT
  Direction(2) = GOLEFT

  Movement = 50

'*
'* now call the procedure that controls the play and pass the position
'* of each Spacoid.
'*
  
  Play Position()

End Sub

'*************************************************************************
'*                                                                       *
'*                Name: Main                                             *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To control the whole program.                    *
'*                                                                       *
'*    Passed variables: None                                             *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes: None                                             *
'*                                                                       *
'*************************************************************************
'*
Sub Main ()

'*
'* Here are the user changeable variables:
'*
'*************************************************************************

  NumberOfPlayers = 2

  ReDim Direction(1 To NumberOfPlayers)
  ReDim Colour(1 To NumberOfPlayers)

  Colour(1) = 14      '* Light Yellow
  Colour(2) = 13      '* Light Magenta
  
  Delay = 1

'*************************************************************************
'*
'* and now, check that the number of players entered doesn't exceed the
'* number we've catered for and show the playing area.
'*

  If NumberOfPlayers > 2 Then
    Answer = UserMessage(1, "")
    QuitRoutine False
  End If

  CentreForm FrmPlay
  FrmPlay.Show MODAL

End Sub

'*************************************************************************
'*                                                                       *
'*                Name: Play                                             *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To start the Spacoids moving and then control    *
'*                      their direction with specific keyboard keys.     *
'*                                                                       *
'*    Passed variables: Ps() - The position of each Spacoid              *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes: There are only four possible directions allowed, *
'*                      Right, Left, Up and Down, and these are declared *
'*                      among the Global Constants.                      *
'*                                                                       *
'*************************************************************************
'*
Sub Play (Ps() As Single)

  Dim Count As Integer
  Dim Which As Integer

  Dim Wait As Double

  Do

'*
'* Move the Spacoids...
'*

    Which = UpdatePosition(Ps(), Direction())

    If Which > 0 Then

'*
'* one of them has crashed so go to the end play...
'*

      EndPlay Which

      Exit Sub

    End If

'*
'* otherwise draw the new position...
'*

    For Count = 1 To NumberOfPlayers
      FrmPlay.BackDrop.PSet (Ps(Count, 0), Ps(Count, 1)), QBColor(Colour(Count))
    Next

'*
'* Slow down the action, if required, by an amount regulated by the value
'* of Delay...
'*

    Wait = Timer

    While Timer < Wait + (Delay / 100)
      DoEvents
    Wend

'*
'* and let Windows do a bit of processing so that any keyboard or mouse
'* inputs will be registered.
'*

    DoEvents

  Loop

End Sub

'*************************************************************************
'*                                                                       *
'*                Name: QuitRoutine                                      *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To exit neatly from the program.                 *
'*                                                                       *
'*    Passed variables: Sh - Show 'are you sure' message if True         *
'*                                                                       *
'*  Returned variables: None                                             *
'*                                                                       *
'*               Notes: None                                             *
'*                                                                       *
'*************************************************************************
'*
Sub QuitRoutine (Sh As Integer)

  If Sh Then
    If UserMessage(0, "") = IDNO Then
      Exit Sub
    End If
  End If

  End
    
End Sub

'*************************************************************************
'*                                                                       *
'*                Name: UpdatePosition                                   *
'*                Type: Procedure                                        *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.00                                             *
'*    Date of Last Mod: 6 September 1995                                 *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To check that the new position of a Spacoid will *
'*                      not be out of the playing area and return the    *
'*                      player number if it is.                          *
'*                                                                       *
'*    Passed variables: Ps() - The position of the Spacoid               *
'*                      Dr() - The direction in which it's travelling    *
'*                                                                       *
'*  Returned variables: UpdatePosition - If all Spacoids are within the  *
'*                                       playing area this will be 0     *
'*                                       The number of the player if a   *
'*                                       Spacoid is going out of bounds  *
'*                                                                       *
'*               Notes:                                                  *
'*                                                                       *
'*************************************************************************
'*
Function UpdatePosition (Ps() As Single, Dr() As Integer) As Integer

  Dim Count As Integer
  
  Dim tArray() As Single

  ReDim tArray(1 To NumberOfPlayers, 0 To 1)

  UpdatePosition = 0

'*
'* Move each Spacoid in turn...
'*

  For Count = 1 To NumberOfPlayers

    tArray(Count, 0) = Ps(Count, 0)
    tArray(Count, 1) = Ps(Count, 1)
    
'*
'* by checking that continuing in their current direction won't cause them
'* to collide with a trail or go out of bounds.
'*

    Select Case Dr(Count)

      Case GORIGHT:

'*
'* Start with any Spacoid that's moving right...
'*

          If Not DetectCollision(Ps(Count, 0) + Movement, Ps(Count, 1)) Then

'*
'* The collision detection function returned False, so or it's about to
'* crash in to a trail or go out of the playing area.
'*

            UpdatePosition = Count
            Exit For
  
          Else

'*
'* The collision detection function returned True so change this Spacoid's
'* location.
'*

            Ps(Count, 0) = Ps(Count, 0) + Movement

          End If

      Case GOLEFT:

'*
'* now check those going left...
'*

          If Not DetectCollision(Ps(Count, 0) - Movement, Ps(Count, 1)) Then
          
            UpdatePosition = Count
            Exit For
  
          Else

            Ps(Count, 0) = Ps(Count, 0) - Movement

          End If

      Case GOUP:

'*
'* going up (household furnishings, lingerie, pet food and ironmongery)...
'*

          If Not DetectCollision(Ps(Count, 0), Ps(Count, 1) - Movement) Then
          
            UpdatePosition = Count
            Exit For
  
          Else

            Ps(Count, 1) = Ps(Count, 1) - Movement

          End If

      Case GODOWN:

'*
'* and down.
'*

          If Not DetectCollision(Ps(Count, 0), Ps(Count, 1) + Movement) Then
          
            UpdatePosition = Count
            Exit For
  
          Else

            Ps(Count, 1) = Ps(Count, 1) + Movement

          End If

    End Select

    DoEvents

  Next

  Erase tArray

End Function

'*************************************************************************
'*                                                                       *
'*                Name: UserMessage                                      *
'*                Type: Function                                         *
'*                                                                       *
'*              Author: Chuck Pettitt                                    *
'*      Version Number: 1.15                                             *
'*    Date of Last Mod: 2 February 1993                                  *
'*      Reason for Mod: None                                             *
'*                                                                       *
'*             Purpose: To display system messages to the user and get   *
'*                      their reply.                                     *
'*                                                                       *
'*    Passed variables: Wh - Which message to show                       *
'*                      St - A string variable used to display dynamic   *
'*                           information                                 *
'*                                                                       *
'*  Returned variables: UserMessage - The number of the reply            *
'*                                                                       *
'*               Notes: See the Global Constants in General|Declarations *
'*                      to find out what the various constants are.      *
'*                                                                       *
'*************************************************************************
'*
Function UserMessage (Wh As Integer, St As String) As Integer

  Dim Message As String
  
  Dim BoxType As Integer

  Select Case Wh
    Case 0:
      Message = "Do you really want to stop playing Spacoids?"
      BoxType = MB_ICONQUESTION + MB_YESNO
    Case 1:
      Message = "The maximum number of players for this version is two."
      BoxType = MB_ICONINFORMATION + MB_OK
  End Select

  Beep
  UserMessage = MsgBox(Message, BoxType, "Spacoids from PC Answers")

End Function

